#!/usr/bin/python
# -*- coding: utf-8 -*-

# Copyright (c) 2021, Cisco Systems
# GNU General Public License v3.0+ (see LICENSE or
# https://www.gnu.org/licenses/gpl-3.0.txt)

DOCUMENTATION = r"""
module: devices_switch_ports
short_description: Resource module for devices _switch _ports
description:
  - Manage operation update of the resource devices _switch _ports.
  - Update a switch port.
version_added: '1.0.0'
extends_documentation_fragment:
  - cisco.meraki.module
author: Francisco Munoz (@fmunoz)
options:
  accessPolicyNumber:
    description: The number of a custom access policy to configure on the switch port.
      Only applicable when 'accessPolicyType' is 'Custom access policy'.
    type: int
  accessPolicyType:
    description: The type of the access policy of the switch port. Only applicable
      to access ports. Can be one of 'Open', 'Custom access policy', 'MAC allow list'
      or 'Sticky MAC allow list'.
    type: str
  adaptivePolicyGroupId:
    description: The adaptive policy group ID that will be used to tag traffic through
      this switch port. This ID must pre-exist during the configuration, else needs
      to be created using adaptivePolicy/groups API. Cannot be applied to a port on
      a switch bound to profile.
    type: str
  allowedVlans:
    description: The VLANs allowed on the switch port. Only applicable to trunk ports.
    type: str
  daiTrusted:
    description: If true, ARP packets for this port will be considered trusted, and
      Dynamic ARP Inspection will allow the traffic.
    type: bool
  dot3az:
    description: Dot3az settings for the port.
    suboptions:
      enabled:
        description: The Energy Efficient Ethernet status of the switch port.
        type: bool
    type: dict
  enabled:
    description: The status of the switch port.
    type: bool
  flexibleStackingEnabled:
    description: For supported switches (e.g. MS420/MS425), whether or not the port
      has flexible stacking enabled.
    type: bool
  highSpeed:
    description: High speed port enablement settings for C9500-32QC.
    suboptions:
      enabled:
        description: For C9500-32QC, whether or not the port is enabled for high speed.
        type: bool
    type: dict
  isolationEnabled:
    description: The isolation status of the switch port.
    type: bool
  linkNegotiation:
    description: The link speed for the switch port.
    type: str
  macAllowList:
    description: Only devices with MAC addresses specified in this list will have
      access to this port. Up to 20 MAC addresses can be defined. Only applicable
      when 'accessPolicyType' is 'MAC allow list'.
    elements: str
    type: list
  macWhitelistLimit:
    description: The maximum number of MAC addresses for regular MAC allow list. Only
      applicable when 'accessPolicyType' is 'MAC allow list'. Note Config only supported
      on verions greater than ms18 only for classic switches.
    type: int
  name:
    description: The name of the switch port.
    type: str
  peerSgtCapable:
    description: If true, Peer SGT is enabled for traffic through this switch port.
      Applicable to trunk port only, not access port. Cannot be applied to a port
      on a switch bound to profile.
    type: bool
  poeEnabled:
    description: The PoE status of the switch port.
    type: bool
  portId:
    description: PortId path parameter. Port ID.
    type: str
  portScheduleId:
    description: The ID of the port schedule. A value of null will clear the port
      schedule.
    type: str
  profile:
    description: Profile attributes.
    suboptions:
      enabled:
        description: When enabled, override this port's configuration with a port
          profile.
        type: bool
      id:
        description: When enabled, the ID of the port profile used to override the
          port's configuration.
        type: str
      iname:
        description: When enabled, the IName of the profile.
        type: str
    type: dict
  rstpEnabled:
    description: The rapid spanning tree protocol status.
    type: bool
  serial:
    description: Serial path parameter.
    type: str
  stickyMacAllowList:
    description: The initial list of MAC addresses for sticky Mac allow list. Only
      applicable when 'accessPolicyType' is 'Sticky MAC allow list'.
    elements: str
    type: list
  stickyMacAllowListLimit:
    description: The maximum number of MAC addresses for sticky MAC allow list. Only
      applicable when 'accessPolicyType' is 'Sticky MAC allow list'.
    type: int
  stormControlEnabled:
    description: The storm control status of the switch port.
    type: bool
  stpGuard:
    description: The state of the STP guard ('disabled', 'root guard', 'bpdu guard'
      or 'loop guard').
    type: str
  stpPortFastTrunk:
    description: The state of STP PortFast Trunk on the switch port.
    type: bool
  tags:
    description: The list of tags of the switch port.
    elements: str
    type: list
  type:
    description: The type of the switch port ('access', 'trunk', 'stack', 'routed',
      'svl' or 'dad').
    type: str
  udld:
    description: The action to take when Unidirectional Link is detected (Alert only,
      Enforce). Default configuration is Alert only.
    type: str
  vlan:
    description: The VLAN of the switch port. For a trunk port, this is the native
      VLAN. A null value will clear the value set for trunk ports.
    type: int
  voiceVlan:
    description: The voice VLAN of the switch port. Only applicable to access ports.
    type: int
requirements:
  - meraki >= 2.4.9
  - python >= 3.5
seealso:
  - name: Cisco Meraki documentation for switch updateDeviceSwitchPort
    description: Complete reference of the updateDeviceSwitchPort API.
    link: https://developer.cisco.com/meraki/api-v1/#!update-device-switch-port
notes:
  - SDK Method used are
    switch.Switch.update_device_switch_port,
  - Paths used are
    put /devices/{serial}/switch/ports/{portId},
"""

EXAMPLES = r"""
- name: Update by id
  cisco.meraki.devices_switch_ports:
    meraki_api_key: "{{ meraki_api_key }}"
    meraki_base_url: "{{ meraki_base_url }}"
    meraki_single_request_timeout: "{{ meraki_single_request_timeout }}"
    meraki_certificate_path: "{{ meraki_certificate_path }}"
    meraki_requests_proxy: "{{ meraki_requests_proxy }}"
    meraki_wait_on_rate_limit: "{{ meraki_wait_on_rate_limit }}"
    meraki_nginx_429_retry_wait_time: "{{ meraki_nginx_429_retry_wait_time }}"
    meraki_action_batch_retry_wait_time: "{{ meraki_action_batch_retry_wait_time }}"
    meraki_retry_4xx_error: "{{ meraki_retry_4xx_error }}"
    meraki_retry_4xx_error_wait_time: "{{ meraki_retry_4xx_error_wait_time }}"
    meraki_maximum_retries: "{{ meraki_maximum_retries }}"
    meraki_output_log: "{{ meraki_output_log }}"
    meraki_log_file_prefix: "{{ meraki_log_file_prefix }}"
    meraki_log_path: "{{ meraki_log_path }}"
    meraki_print_console: "{{ meraki_print_console }}"
    meraki_suppress_logging: "{{ meraki_suppress_logging }}"
    meraki_simulate: "{{ meraki_simulate }}"
    meraki_be_geo_id: "{{ meraki_be_geo_id }}"
    meraki_caller: "{{ meraki_caller }}"
    meraki_use_iterator_for_get_pages: "{{ meraki_use_iterator_for_get_pages }}"
    meraki_inherit_logging_config: "{{ meraki_inherit_logging_config }}"
    state: present
    accessPolicyNumber: 2
    accessPolicyType: Sticky MAC allow list
    adaptivePolicyGroupId: '123'
    allowedVlans: 1,3,5-10
    daiTrusted: false
    dot3az:
      enabled: false
    enabled: true
    flexibleStackingEnabled: true
    highSpeed:
      enabled: false
    isolationEnabled: false
    linkNegotiation: Auto negotiate
    macAllowList:
      - 34:56:fe:ce:8e:a0
      - 34:56:fe:ce:8e:a1
    macWhitelistLimit: 10
    name: My switch port
    peerSgtCapable: false
    poeEnabled: true
    portId: string
    portScheduleId: '1234'
    profile:
      enabled: false
      id: '1284392014819'
      iname: iname
    rstpEnabled: true
    serial: string
    stickyMacAllowList:
      - 34:56:fe:ce:8e:b0
      - 34:56:fe:ce:8e:b1
    stickyMacAllowListLimit: 5
    stormControlEnabled: true
    stpGuard: disabled
    stpPortFastTrunk: false
    tags:
      - tag1
      - tag2
    type: access
    udld: Alert only
    vlan: 10
    voiceVlan: 20
"""
RETURN = r"""
meraki_response:
  description: A dictionary or list with the response returned by the Cisco Meraki Python SDK
  returned: always
  type: dict
  sample: >
    {
      "portId": "string",
      "name": "string",
      "tags": [
        "string"
      ],
      "enabled": true,
      "poeEnabled": true,
      "type": "string",
      "vlan": 0,
      "voiceVlan": 0,
      "allowedVlans": "string",
      "isolationEnabled": true,
      "rstpEnabled": true,
      "stpGuard": "string",
      "stpPortFastTrunk": true,
      "linkNegotiation": "string",
      "linkNegotiationCapabilities": [
        "string"
      ],
      "portScheduleId": "string",
      "schedule": {
        "id": "string",
        "name": "string"
      },
      "udld": "string",
      "accessPolicyType": "string",
      "accessPolicyNumber": 0,
      "macAllowList": [
        "string"
      ],
      "macWhitelistLimit": 0,
      "stickyMacAllowList": [
        "string"
      ],
      "stickyMacAllowListLimit": 0,
      "stormControlEnabled": true,
      "adaptivePolicyGroupId": "string",
      "adaptivePolicyGroup": {
        "id": "string",
        "name": "string"
      },
      "peerSgtCapable": true,
      "flexibleStackingEnabled": true,
      "daiTrusted": true,
      "profile": {
        "enabled": true,
        "id": "string",
        "iname": "string"
      },
      "module": {
        "model": "string",
        "serial": "string",
        "slot": 0
      },
      "mirror": {
        "mode": "string"
      },
      "dot3az": {
        "enabled": true
      },
      "highSpeed": {
        "enabled": true
      }
    }
"""



#!/usr/bin/python
# -*- coding: utf-8 -*-

# Copyright: (c) 2023, Gaspard Micol (@gmicol) <gmicol@cisco.com>
# GNU General Public License v3.0+ (see LICENSE or https://www.gnu.org/licenses/gpl-3.0.txt)

from __future__ import absolute_import, division, print_function

__metaclass__ = type

ANSIBLE_METADATA = {"metadata_version": "1.1", "status": ["preview"], "supported_by": "community"}

DOCUMENTATION = r"""
---
module: aci_maintenance_policy
short_description: Manage firmware maintenance policies (maint:MaintP)
description:
- Manage maintenance policies that defines behavior during an ACI upgrade.
options:
  name:
    description:
    - The name for the maintenance policy.
    type: str
    aliases: [ maintenance_policy ]
  run_mode:
    description:
    - Whether the system pauses on error or just continues through it.
    - The APIC defaults to C(pauseOnlyOnFailures) when unset during creation.
    type: str
    choices: [ pause_always_between_sets, pause_only_on_failures, pause_never, pauseOnlyOnFailures, pauseNever ]
    aliases: [ runmode ]
  graceful:
    description:
    - Whether the system will bring down the nodes gracefully during an upgrade, which reduces traffic lost.
    - The APIC defaults to C(false) when unset during creation.
    type: bool
  scheduler:
    description:
    - The name of scheduler that is applied to the policy.
    type: str
  admin_state:
    description:
    - The administrative state of the executable policies.
    - Will trigger an immediate upgrade for nodes if C(admin_state) is set to triggered.
    - The APIC defaults to C(untriggered) when unset during creation.
    type: str
    choices: [ triggered, untriggered ]
    aliases: [ adminst ]
  download_state:
    description:
    - The download state of the executable policies.
    - The APIC defaults to C(untriggered) when unset during creation.
    type: str
    choices: [ triggered, untriggered ]
  notify_condition:
    description:
    - Specifies under what pause condition will admin be notified via email/text as configured.
    - This notification mechanism is independent of events/faults.
    - The APIC defaults to C(notifyOnlyOnFailures) when unset during creation.
    type: str
    choices: [ notify_always_between_sets, notify_never, notify_only_on_failures ]
  smu_operation:
    description:
    - Specifies that the upgrade is a Software Maintenance Upgrade (SMU) patch operation.
    type: str
    choices: [ smu_install, smu_uninstall ]
  smu_operation_flags:
    description:
    - Specifies the Software Maintenance Upgrade (SMU) patch operation flags
    - Indicates if node should be reloaded immediately or skip auto reload on SMU Install/Uninstall.
    type: str
    choices: [ smu_reload_immediate, smu_reload_skip ]
  sr_upgrade:
    description:
    - Specifies that the upgrade is a Silent Roll (SR) package upgrade.
    type: bool
    aliases: [ silent_roll_upgrade ]
  sr_version:
    description:
    - The target firmware version of the Silent Roll (SR) package upgrade to install.
    type: str
    aliases: [ silent_roll_version ]
  version:
    description:
    - The target firmware version to install.
    type: str
  version_check_override:
    description:
    - The version check override.
    - This is a directive to ignore the version check for the next install.
    - The version check, which occurs during a maintenance window, checks to see if the desired version matches the running version.
    - If the versions do not match, the install is performed. If the versions do match, the install is not performed.
    - The version check override is a one-time override that performs the install whether or not the versions match.
    - The APIC defaults to C(untriggered) when unset during creation.
    type: str
    choices: [ trigger, trigger_immediate, triggered, untriggered ]
  ignore_compat:
    description:
    - To check whether compatibility checks should be ignored
    - The APIC defaults to C(false) when unset during creation.
    type: bool
    aliases: [ ignoreCompat ]
  description:
    description:
    - Description for the maintenance policy.
    type: str
    aliases: [ descr ]
  state:
    description:
    - Use C(present) or C(absent) for adding or removing.
    - Use C(query) for listing an object or multiple objects.
    type: str
    choices: [ absent, present, query ]
    default: present
  name_alias:
    description:
    - The alias for the current object. This relates to the nameAlias field in ACI.
    type: str
extends_documentation_fragment:
- cisco.aci.aci
- cisco.aci.annotation

notes:
- A scheduler is required for this module, which could have been created using the M(cisco.aci.aci_fabric_scheduler) module or via the UI.
seealso:
- module: cisco.aci.aci_fabric_scheduler
- name: APIC Management Information Model reference
  description: More information about the internal APIC class B(maint:MaintP).
  link: https://developer.cisco.com/docs/apic-mim-ref/
author:
- Steven Gerhart (@sgerhart)
- Gaspard Micol (@gmicol)
"""

EXAMPLES = r"""
- name: Create a maintenance policy
  cisco.aci.aci_maintenance_policy:
    host: apic
    username: admin
    password: SomeSecretPassword
    name: my_maintenance_policy
    scheduler: simpleScheduler
    state: present
  delegate_to: localhost

- name: Delete a maintenance policy
  cisco.aci.aci_maintenance_policy:
    host: apic
    username: admin
    password: SomeSecretPassword
    name: my_maintenance_policy
    state: absent
  delegate_to: localhost

- name: Query all maintenance policies
  cisco.aci.aci_maintenance_policy:
    host: apic
    username: admin
    password: SomeSecretPassword
    state: query
  delegate_to: localhost
  register: query_result

- name: Query a specific maintenance policy
  cisco.aci.aci_maintenance_policy:
    host: apic
    username: admin
    password: SomeSecretPassword
    name: my_maintenance_policy
    state: query
  delegate_to: localhost
  register: query_result
"""

RETURN = r"""
current:
  description: The existing configuration from the APIC after the module has finished
  returned: success
  type: list
  sample:
    [
        {
            "fvTenant": {
                "attributes": {
                    "descr": "Production environment",
                    "dn": "uni/tn-production",
                    "name": "production",
                    "nameAlias": "",
                    "ownerKey": "",
                    "ownerTag": ""
                }
            }
        }
    ]
error:
  description: The error information as returned from the APIC
  returned: failure
  type: dict
  sample:
    {
        "code": "122",
        "text": "unknown managed object class foo"
    }
raw:
  description: The raw output returned by the APIC REST API (xml or json)
  returned: parse error
  type: str
  sample: '<?xml version="1.0" encoding="UTF-8"?><imdata totalCount="1"><error code="122" text="unknown managed object class foo"/></imdata>'
sent:
  description: The actual/minimal configuration pushed to the APIC
  returned: info
  type: list
  sample:
    {
        "fvTenant": {
            "attributes": {
                "descr": "Production environment"
            }
        }
    }
previous:
  description: The original configuration from the APIC before the module has started
  returned: info
  type: list
  sample:
    [
        {
            "fvTenant": {
                "attributes": {
                    "descr": "Production",
                    "dn": "uni/tn-production",
                    "name": "production",
                    "nameAlias": "",
                    "ownerKey": "",
                    "ownerTag": ""
                }
            }
        }
    ]
proposed:
  description: The assembled configuration from the user-provided parameters
  returned: info
  type: dict
  sample:
    {
        "fvTenant": {
            "attributes": {
                "descr": "Production environment",
                "name": "production"
            }
        }
    }
filter_string:
  description: The filter string used for the request
  returned: failure or debug
  type: str
  sample: ?rsp-prop-include=config-only
method:
  description: The HTTP method used for the request to the APIC
  returned: failure or debug
  type: str
  sample: POST
response:
  description: The HTTP response from the APIC
  returned: failure or debug
  type: str
  sample: OK (30 bytes)
status:
  description: The HTTP status from the APIC
  returned: failure or debug
  type: int
  sample: 200
url:
  description: The HTTP url used for the request to the APIC
  returned: failure or debug
  type: str
  sample: https://10.11.12.13/api/mo/uni/tn-production.json
"""


from ansible_collections.cisco.aci.plugins.module_utils.aci import ACIModule, aci_argument_spec, aci_annotation_spec
from ansible.module_utils.basic import AnsibleModule
from ansible_collections.cisco.aci.plugins.module_utils.constants import (
    MATCH_RUN_MODE_MAPPING,
    MATCH_NOTIFY_CONDITION_MAPPING,
    MATCH_SMU_OPERATION_MAPPING,
    MATCH_SMU_OPERATION_FLAGS_MAPPING,
    MATCH_TRIGGER_MAPPING,
)


def main():
    list_run_mode_choices = list(MATCH_RUN_MODE_MAPPING.keys())
    list_run_mode_choices.extend(["pauseOnlyOnFailures", "pauseNever"])
    argument_spec = aci_argument_spec()
    argument_spec.update(aci_annotation_spec())
    argument_spec.update(
        name=dict(type="str", aliases=["maintenance_policy"]),  # Not required for querying all objects
        run_mode=dict(type="str", choices=list_run_mode_choices, aliases=["runmode"]),
        graceful=dict(type="bool"),
        scheduler=dict(type="str"),
        ignore_compat=dict(type="bool", aliases=["ignoreCompat"]),
        admin_state=dict(type="str", choices=list(MATCH_TRIGGER_MAPPING.keys())[2:], aliases=["adminst"]),
        download_state=dict(type="str", choices=list(MATCH_TRIGGER_MAPPING.keys())[2:]),
        notify_condition=dict(type="str", choices=list(MATCH_NOTIFY_CONDITION_MAPPING.keys())),
        smu_operation=dict(type="str", choices=list(MATCH_SMU_OPERATION_MAPPING.keys())),
        smu_operation_flags=dict(type="str", choices=list(MATCH_SMU_OPERATION_FLAGS_MAPPING.keys())),
        sr_upgrade=dict(type="bool", aliases=["silent_roll_upgrade"]),
        sr_version=dict(type="str", aliases=["silent_roll_version"]),
        version=dict(type="str"),
        version_check_override=dict(type="str", choices=list(MATCH_TRIGGER_MAPPING.keys())),
        description=dict(type="str", aliases=["descr"]),
        state=dict(type="str", default="present", choices=["absent", "present", "query"]),
        name_alias=dict(type="str"),
    )

    module = AnsibleModule(
        argument_spec=argument_spec,
        supports_check_mode=True,
        required_if=[
            ["state", "absent", ["name"]],
            ["state", "present", ["name", "scheduler"]],
        ],
    )

    aci = ACIModule(module)

    state = module.params.get("state")
    name = module.params.get("name")
    run_mode = module.params.get("run_mode")
    graceful = aci.boolean(module.params.get("graceful"), "yes", "no")
    scheduler = module.params.get("scheduler")
    admin_state = module.params.get("admin_state")
    download_state = module.params.get("download_state")
    notify_condition = MATCH_NOTIFY_CONDITION_MAPPING.get(module.params.get("notify_condition"))
    smu_operation = MATCH_SMU_OPERATION_MAPPING.get(module.params.get("smu_operation"))
    smu_operation_flags = MATCH_SMU_OPERATION_FLAGS_MAPPING.get(module.params.get("smu_operation_flags"))
    sr_version = module.params.get("sr_version")
    sr_upgrade = module.params.get("sr_upgrade")
    version = module.params.get("version")
    version_check_override = MATCH_TRIGGER_MAPPING.get(module.params.get("version_check_override"))
    ignore_compat = aci.boolean(module.params.get("ignore_compat"))
    description = module.params.get("description")
    name_alias = module.params.get("name_alias")

    if run_mode not in ["pauseOnlyOnFailures", "pauseNever"]:
        run_mode = MATCH_RUN_MODE_MAPPING.get(run_mode)

    aci.construct_url(
        root_class=dict(
            aci_class="maintMaintP",
            aci_rn="fabric/maintpol-{0}".format(name),
            target_filter={"name": name},
            module_object=name,
        ),
        child_classes=["maintRsPolScheduler"],
    )

    aci.get_existing()

    if state == "present":
        aci.payload(
            aci_class="maintMaintP",
            class_config=dict(
                name=name,
                descr=description,
                runMode=run_mode,
                graceful=graceful,
                adminSt=admin_state,
                downloadSt=download_state,
                notifCond=notify_condition,
                smuOperation=smu_operation,
                smuOperationFlags=smu_operation_flags,
                srUpgrade=sr_upgrade,
                srVersion=sr_version,
                version=version,
                versionCheckOverride=version_check_override,
                ignoreCompat=ignore_compat,
                nameAlias=name_alias,
            ),
            child_configs=[
                dict(
                    maintRsPolScheduler=dict(
                        attributes=dict(
                            tnTrigSchedPName=scheduler,
                        ),
                    ),
                ),
            ],
        )

        aci.get_diff(aci_class="maintMaintP")

        aci.post_config()

    elif state == "absent":
        aci.delete_config()

    aci.exit_json()


if __name__ == "__main__":
    main()



# This code is part of the Ansible collection community.docker, but is an independent component.
# This particular file, and this file only, is based on the Docker SDK for Python (https://github.com/docker/docker-py/)
#
# Copyright (c) 2016-2022 Docker, Inc.
#
# It is licensed under the Apache 2.0 license (see LICENSES/Apache-2.0.txt in this collection)
# SPDX-License-Identifier: Apache-2.0

# Note that this module util is **PRIVATE** to the collection. It can have breaking changes at any time.
# Do not use this from other collections or standalone plugins/modules!

from __future__ import annotations

import errno
import os
import select
import socket as pysocket
import struct
import typing as t

from ..transport.npipesocket import NpipeSocket

if t.TYPE_CHECKING:
    from collections.abc import Sequence

    from ..._socket_helper import SocketLike


STDOUT = 1
STDERR = 2


class SocketError(Exception):
    pass


# NpipeSockets have their own error types
# pywintypes.error: (109, 'ReadFile', 'The pipe has been ended.')
NPIPE_ENDED = 109


def read(socket: SocketLike, n: int = 4096) -> bytes | None:
    """
    Reads at most n bytes from socket
    """

    recoverable_errors = (errno.EINTR, errno.EDEADLK, errno.EWOULDBLOCK)

    if not isinstance(socket, NpipeSocket):  # type: ignore[unreachable]
        if not hasattr(select, "poll"):
            # Limited to 1024
            select.select([socket], [], [])
        else:
            poll = select.poll()
            poll.register(socket, select.POLLIN | select.POLLPRI)
            poll.poll()

    try:
        if hasattr(socket, "recv"):
            return socket.recv(n)
        if isinstance(socket, pysocket.SocketIO):  # type: ignore
            return socket.read(n)  # type: ignore[unreachable]
        return os.read(socket.fileno(), n)
    except EnvironmentError as e:
        if e.errno not in recoverable_errors:
            raise
        return None  # TODO ???
    except Exception as e:
        is_pipe_ended = (
            isinstance(socket, NpipeSocket)  # type: ignore[unreachable]
            and len(e.args) > 0
            and e.args[0] == NPIPE_ENDED
        )
        if is_pipe_ended:
            # npipes do not support duplex sockets, so we interpret
            # a PIPE_ENDED error as a close operation (0-length read).
            return b""
        raise


def read_exactly(socket: SocketLike, n: int) -> bytes:
    """
    Reads exactly n bytes from socket
    Raises SocketError if there is not enough data
    """
    data = b""
    while len(data) < n:
        next_data = read(socket, n - len(data))
        if not next_data:
            raise SocketError("Unexpected EOF")
        data += next_data
    return data


def next_frame_header(socket: SocketLike) -> tuple[int, int]:
    """
    Returns the stream and size of the next frame of data waiting to be read
    from socket, according to the protocol defined here:

    https://docs.docker.com/engine/api/v1.24/#attach-to-a-container
    """
    try:
        data = read_exactly(socket, 8)
    except SocketError:
        return (-1, -1)

    stream, actual = struct.unpack(">BxxxL", data)
    return (stream, actual)


def frames_iter(socket: SocketLike, tty: bool) -> t.Generator[tuple[int, bytes]]:
    """
    Return a generator of frames read from socket. A frame is a tuple where
    the first item is the stream number and the second item is a chunk of data.

    If the tty setting is enabled, the streams are multiplexed into the stdout
    stream.
    """
    if tty:
        return ((STDOUT, frame) for frame in frames_iter_tty(socket))
    return frames_iter_no_tty(socket)


def frames_iter_no_tty(socket: SocketLike) -> t.Generator[tuple[int, bytes]]:
    """
    Returns a generator of data read from the socket when the tty setting is
    not enabled.
    """
    while True:
        stream, n = next_frame_header(socket)
        if n < 0:
            break
        while n > 0:
            result = read(socket, n)
            if result is None:
                continue
            data_length = len(result)
            if data_length == 0:
                # We have reached EOF
                return
            n -= data_length
            yield (stream, result)


def frames_iter_tty(socket: SocketLike) -> t.Generator[bytes]:
    """
    Return a generator of data read from the socket when the tty setting is
    enabled.
    """
    while True:
        result = read(socket)
        if not result:
            # We have reached EOF
            return
        yield result


@t.overload
def consume_socket_output(
    frames: Sequence[bytes] | t.Generator[bytes], demux: t.Literal[False] = False
) -> bytes: ...


@t.overload
def consume_socket_output(
    frames: (
        Sequence[tuple[bytes | None, bytes | None]]
        | t.Generator[tuple[bytes | None, bytes | None]]
    ),
    demux: t.Literal[True],
) -> tuple[bytes, bytes]: ...


@t.overload
def consume_socket_output(
    frames: (
        Sequence[bytes]
        | Sequence[tuple[bytes | None, bytes | None]]
        | t.Generator[bytes]
        | t.Generator[tuple[bytes | None, bytes | None]]
    ),
    demux: bool = False,
) -> bytes | tuple[bytes, bytes]: ...


def consume_socket_output(
    frames: (
        Sequence[bytes]
        | Sequence[tuple[bytes | None, bytes | None]]
        | t.Generator[bytes]
        | t.Generator[tuple[bytes | None, bytes | None]]
    ),
    demux: bool = False,
) -> bytes | tuple[bytes, bytes]:
    """
    Iterate through frames read from the socket and return the result.

    Args:

        demux (bool):
            If False, stdout and stderr are multiplexed, and the result is the
            concatenation of all the frames. If True, the streams are
            demultiplexed, and the result is a 2-tuple where each item is the
            concatenation of frames belonging to the same stream.
    """
    if demux is False:
        # If the streams are multiplexed, the generator returns strings, that
        # we just need to concatenate.
        return b"".join(frames)  # type: ignore

    # If the streams are demultiplexed, the generator yields tuples
    # (stdout, stderr)
    out: list[bytes | None] = [None, None]
    frame: tuple[bytes | None, bytes | None]
    for frame in frames:  # type: ignore
        # It is guaranteed that for each frame, one and only one stream
        # is not None.
        if frame == (None, None):
            raise AssertionError(f"frame must be (None, None), but got {frame}")
        if frame[0] is not None:
            if out[0] is None:
                out[0] = frame[0]
            else:
                out[0] += frame[0]
        else:
            if out[1] is None:
                out[1] = frame[1]
            else:
                out[1] += frame[1]  # type: ignore[operator]
    return tuple(out)  # type: ignore


def demux_adaptor(stream_id: int, data: bytes) -> tuple[bytes | None, bytes | None]:
    """
    Utility to demultiplex stdout and stderr when reading frames from the
    socket.
    """
    if stream_id == STDOUT:
        return (data, None)
    if stream_id == STDERR:
        return (None, data)
    raise ValueError(f"{stream_id} is not a valid stream")



#!/usr/bin/python
# -*- coding: utf-8 -*-
# Copyright 2019 Red Hat
# GNU General Public License v3.0+
# (see COPYING or https://www.gnu.org/licenses/gpl-3.0.txt)

#############################################
#                WARNING                    #
#############################################
#
# This file is auto generated by the resource
#   module builder playbook.
#
# Do not edit this file manually.
#
# Changes to this file will be over written
#   by the resource module builder.
#
# Changes should be made in the model used to
#   generate this file or in the resource module
#   builder template.
#
#############################################
"""
The module file for nxos_lldp_global
"""
from __future__ import absolute_import, division, print_function


__metaclass__ = type


DOCUMENTATION = """
module: nxos_lldp_global
short_description: LLDP resource module
description: This module configures and manages the Link Layer Discovery Protocol(LLDP)
  attributes on NX-OS platforms.
version_added: 1.0.0
author: Adharsh Srivats Rangarajan (@adharshsrivatsr)
notes:
- Tested against NxOS 7.3.(0)D1(1) on VIRL
- Unsupported for Cisco MDS
- The LLDP feature needs to be enabled before using this module
options:
  running_config:
    description:
    - This option is used only with state I(parsed).
    - The value of this option should be the output received from the NX-OS device
      by executing the command B(show running-config | include lldp).
    - The state I(parsed) reads the configuration from C(running_config) option and
      transforms it into Ansible structured data as per the resource module's argspec
      and the value is then returned in the I(parsed) key within the result.
    type: str
  config:
    description:
    - A list of link layer discovery configurations
    type: dict
    suboptions:
      holdtime:
        description:
        - Amount of time the receiving device should hold the information (in seconds)
        type: int
      port_id:
        description:
        - This attribute defines if the interface names should be advertised in the
          long(0) or short(1) form.
        type: int
        choices:
        - 0
        - 1
      reinit:
        description:
        - Amount of time to delay the initialization of LLDP on any interface (in
          seconds)
        type: int
      timer:
        description:
        - Frequency at which LLDP updates need to be transmitted (in seconds)
        type: int
      tlv_select:
        description:
        - This attribute can be used to specify the TLVs that need to be sent and
          received in the LLDP packets. By default, all TLVs are advertised
        type: dict
        suboptions:
          dcbxp:
            description:
            - Used to specify the Data Center Bridging Exchange Protocol TLV
            type: bool
          management_address:
            description:
            - Used to specify the management address in TLV messages
            type: dict
            suboptions:
              v4:
                description: Management address with TLV v4
                type: bool
              v6:
                description: Management address with TLV v6
                type: bool
          port:
            description:
            - Used to manage port based attributes in TLV messages
            type: dict
            suboptions:
              description:
                description:
                - Used to specify the port description TLV
                type: bool
              vlan:
                description:
                - Used to specify the port VLAN ID TLV
                type: bool
          power_management:
            description:
            - Used to specify IEEE 802.3 DTE Power via MDI TLV
            type: bool
          system:
            description:
            - Used to manage system based attributes in TLV messages
            type: dict
            suboptions:
              capabilities:
                description:
                - Used to specify the system capabilities TLV
                type: bool
              description:
                description:
                - Used to specify the system description TLV
                type: bool
              name:
                description:
                - Used to specify the system name TLV
                type: bool
  state:
    description:
    - The state of the configuration after module completion.
    - States C(replaced) and C(overridden) have the same behaviour for this module.
    type: str
    choices:
    - merged
    - replaced
    - overridden
    - deleted
    - gathered
    - rendered
    - parsed
    default: merged

"""
EXAMPLES = """
# Using merged
# Before state:
# -------------
#
# user(config)# show running-config | include lldp
# feature lldp

- name: Merge provided configuration with device configuration
  cisco.nxos.nxos_lldp_global:
    config:
      timer: 35
      holdtime: 100
    state: merged

# After state:
# ------------
#
# user(config)# show running-config | include lldp
# feature lldp
# lldp timer 35
# lldp holdtime 100


# Using replaced
# Before state:
# -------------
#
# user(config)# show running-config | include lldp
# feature lldp
# lldp holdtime 100
# lldp reinit 5
# lldp timer 35

- name: Replace device configuration of specific LLDP attributes with provided configuration
  cisco.nxos.nxos_lldp_global:
    config:
      timer: 40
      tlv_select:
        system:
          description: true
          name: false
        management_address:
          v4: true
    state: replaced

# After state:
# ------------
#
# user(config)# show running-config | include lldp
# feature lldp
# lldp timer 40
# no lldp tlv-select system-name


# Using deleted
# Before state:
# -------------
#
# user(config)# show running-config | include lldp
# feature lldp
# lldp holdtime 5
# lldp reinit 3

- name: Delete LLDP configuration (this will by default remove all lldp configuration)
  cisco.nxos.nxos_lldp_global:
    state: deleted

# After state:
# ------------
#
# user(config)# show running-config | include lldp
# feature lldp

# Using rendered

- name: Use rendered state to convert task input to device specific commands
  cisco.nxos.nxos_lldp_global:
    config:
      holdtime: 130
      port_id: 1
      reinit: 5
      tlv_select:
        dcbxp: true
        power_management: true
    state: rendered

# Task Output (redacted)
# -----------------------

# rendered:
#   - "lldp tlv-select dcbxp"
#   - "lldp tlv-select power-management"
#   - "lldp portid-subtype 1"
#   - "lldp reinit 5"
#   - "lldp holdtime 130"

# Using parsed

# parsed.cfg
# ------------
# lldp holdtime 131
# lldp reinit 7
# no lldp tlv-select system-name
# no lldp tlv-select system-description

# Task output (redacted)
# -----------------------

# parsed:
#   holdtime: 131
#   reinit: 7
#   tlv_select:
#     system:
#       description: false
#       name: false

# Using gathered

# Existing device config state
# -------------------------------
# feature lldp
# lldp holdtime 129
# lldp reinit 5
# lldp timer 35
# no lldp tlv-select system-name

# Task output (redacted)
# -----------------------

# gathered:
#   reinit: 5
#   timer: 35
#   tlv_select:
#     system:
#       name: False
#   holdtime: 129
"""
RETURN = """
before:
  description: The configuration as structured data prior to module invocation.
  returned: always
  type: dict
  sample: >
    The configuration returned will always be in the same format
     of the parameters above.
after:
  description: The configuration as structured data after module completion.
  returned: when changed
  type: dict
  sample: >
    The configuration returned will always be in the same format
     of the parameters above.
commands:
  description: The set of commands pushed to the remote device.
  returned: always
  type: list
  sample: ['lldp holdtime 125', 'lldp reinit 4', 'no lldp tlv-select system-name']
"""

from ansible.module_utils.basic import AnsibleModule

from ansible_collections.cisco.nxos.plugins.module_utils.network.nxos.argspec.lldp_global.lldp_global import (
    Lldp_globalArgs,
)
from ansible_collections.cisco.nxos.plugins.module_utils.network.nxos.config.lldp_global.lldp_global import (
    Lldp_global,
)


def main():
    """
    Main entry point for module execution

    :returns: the result form module invocation
    """
    required_if = [
        ("state", "merged", ("config",)),
        ("state", "replaced", ("config",)),
        ("state", "rendered", ("config",)),
        ("state", "parsed", ("running_config",)),
    ]
    mutually_exclusive = [("config", "running_config")]

    module = AnsibleModule(
        argument_spec=Lldp_globalArgs.argument_spec,
        required_if=required_if,
        mutually_exclusive=mutually_exclusive,
        supports_check_mode=True,
    )

    result = Lldp_global(module).execute_module()
    module.exit_json(**result)


if __name__ == "__main__":
    main()



#!/usr/bin/python
# -*- coding: utf-8 -*-

# GNU General Public License v3.0+ (see COPYING or https://www.gnu.org/licenses/gpl-3.0.txt)

from __future__ import absolute_import, division, print_function
__metaclass__ = type

ANSIBLE_METADATA = {'metadata_version': '1.1',
                    'status': ['preview'],
                    'supported_by': 'community'}

DOCUMENTATION = r'''
---
module: intersight_smtp_policy
short_description: SMTP Policy configuration for Cisco Intersight
description:
  - Manages SMTP Policy configuration on Cisco Intersight.
  - Configures SMTP settings for email notifications.
  - For more information see L(Cisco Intersight,https://intersight.com/apidocs).
extends_documentation_fragment: intersight
options:
  state:
    description:
      - If C(present), will verify the resource is present and will create if needed.
      - If C(absent), will verify the resource is absent and will delete if needed.
    type: str
    choices: [present, absent]
    default: present
  organization:
    description:
      - The name of the Organization this resource is assigned to.
      - Profiles, Policies, and Pools that are created within a Custom Organization are applicable only to devices in the same Organization.
    type: str
    default: default
  name:
    description:
      - The name assigned to the SMTP Policy.
      - The name must be between 1 and 64 alphanumeric characters, allowing special characters :-_.
    type: str
    required: true
  description:
    description:
      - The user-defined description for the SMTP Policy.
      - Description can contain letters(a-z, A-Z), numbers(0-9), hyphen(-), period(.), colon(:), or an underscore(_).
    type: str
    aliases: [descr]
  tags:
    description:
      - List of tags in Key:<user-defined key> Value:<user-defined value> format.
    type: list
    elements: dict
    default: []
  enabled:
    description:
      - If enabled, controls the state of the SMTP client service on the managed device.
    type: bool
    default: true
  smtp_server:
    description:
      - IP address or hostname of the SMTP server.
      - The SMTP server is used by the managed device to send email notifications.
      - Required when C(enabled) is True.
    type: str
  smtp_port:
    description:
      - Port number used by the SMTP server for outgoing SMTP communication.
      - Valid range is 1-65535.
    type: int
    default: 25
  min_severity:
    description:
      - Minimum fault severity level to receive email notifications.
      - Email notifications are sent for all faults whose severity is equal to or greater than the chosen level.
    type: str
    choices: [critical, major, minor, warning, condition, condition]
    default: critical
  sender_email:
    description:
      - The email address entered here will be displayed as the from address (mail received from address) of all the SMTP mail alerts that are received.
      - If not configured, the hostname of the server is used in the from address field.
    type: str
  smtp_recipients:
    description:
      - List of email addresses that will receive notifications for faults.
    type: list
    elements: str
author:
  - Ron Gershburg (@rgershbu)
'''

EXAMPLES = r'''
- name: Create SMTP Policy
  cisco.intersight.intersight_smtp_policy:
    api_private_key: "{{ api_private_key }}"
    api_key_id: "{{ api_key_id }}"
    organization: "default"
    name: "smtp-policy-test"
    description: "SMTP Policy for Alerting"
    enabled: true
    smtp_server: "smtp.example.com"
    smtp_port: 25
    min_severity: "warning"
    sender_email: "alerts@example.com"
    smtp_recipients:
      - "admin@example.com"
      - "devops@example.com"
    tags:
      - Key: "Environment"
        Value: "Production"
    state: present

- name: Disable SMTP Policy
  cisco.intersight.intersight_smtp_policy:
    api_private_key: "{{ api_private_key }}"
    api_key_id: "{{ api_key_id }}"
    name: "smtp-policy-test"
    enabled: false
    state: present

- name: Delete SMTP Policy
  cisco.intersight.intersight_smtp_policy:
    api_private_key: "{{ api_private_key }}"
    api_key_id: "{{ api_key_id }}"
    name: "smtp-policy-test"
    state: absent
'''

RETURN = r'''
api_response:
  description: The API response output returned by the specified resource.
  returned: always
  type: dict
  sample:
    "api_response": {
        "Name": "smtp-policy-test",
        "ObjectType": "smtp.Policy",
        "Description": "SMTP Policy for Alerting",
        "Enabled": true,
        "SmtpServer": "smtp.example.com",
        "SmtpPort": 25,
        "MinSeverity": "warning",
        "SenderEmail": "alerts@example.com",
        "SmtpRecipients": [
            "admin@example.com",
            "devops@example.com"
        ],
        "Tags": [
            {
                "Key": "Environment",
                "Value": "Production"
            }
        ]
    }
'''

from ansible.module_utils.basic import AnsibleModule
from ansible_collections.cisco.intersight.plugins.module_utils.intersight import IntersightModule, intersight_argument_spec


def main():
    argument_spec = intersight_argument_spec.copy()
    argument_spec.update(
        state=dict(type='str', choices=['present', 'absent'], default='present'),
        organization=dict(type='str', default='default'),
        name=dict(type='str', required=True),
        description=dict(type='str', aliases=['descr']),
        tags=dict(type='list', elements='dict', default=[]),
        enabled=dict(type='bool', default=True),
        smtp_server=dict(type='str'),
        smtp_port=dict(type='int', default=25),
        min_severity=dict(type='str', choices=['critical', 'major', 'minor', 'warning', 'condition', 'condition'], default='critical'),
        sender_email=dict(type='str'),
        smtp_recipients=dict(type='list', elements='str')
    )

    module = AnsibleModule(
        argument_spec,
        supports_check_mode=True,
    )

    intersight = IntersightModule(module)
    intersight.result['api_response'] = {}
    intersight.result['trace_id'] = ''

    # Build API Body
    intersight.api_body = {
        'Organization': {
            'Name': intersight.module.params['organization'],
        },
        'Name': intersight.module.params['name']
    }

    if module.params['state'] == 'present':
        intersight.set_tags_and_description()
        intersight.api_body['Enabled'] = module.params['enabled']
        if module.params['enabled']:
            if not module.params.get('smtp_server'):
                module.fail_json(msg="smtp_server is required when enabled is True")

            if module.params.get('smtp_server'):
                intersight.api_body['SmtpServer'] = module.params['smtp_server']
            if module.params.get('smtp_port'):
                intersight.api_body['SmtpPort'] = module.params['smtp_port']
            if module.params.get('min_severity'):
                intersight.api_body['MinSeverity'] = module.params['min_severity']
            if module.params.get('sender_email'):
                intersight.api_body['SenderEmail'] = module.params['sender_email']
            if module.params.get('smtp_recipients'):
                intersight.api_body['SmtpRecipients'] = module.params['smtp_recipients']

    resource_path = '/smtp/Policies'
    intersight.configure_policy_or_profile(resource_path=resource_path)

    module.exit_json(**intersight.result)


if __name__ == '__main__':
    main()



#!/usr/bin/python
#
# GNU General Public License v3.0+ (see COPYING or https://www.gnu.org/licenses/gpl-3.0.txt)

from __future__ import absolute_import, division, print_function
__metaclass__ = type

DOCUMENTATION = '''
---
module: azure_rm_ml_datastore_info
version_added: "3.16.0"
short_description: List ML Data resources
description:
    - List ML Data resources.
options:
    name:
        description:
            - Name of the datastore.
        type: str
        required: false
    resource_group:
        description:
            - Name of resource group.
        required: true
        type: str
    ml_workspace:
        description:
            - Name of the Azure ML workspace.
        required: true
        type: str

extends_documentation_fragment:
    - azure.azcollection.azure

author:
    - Bill Peck (@p3ck)
'''

EXAMPLES = '''
- name: List Specific ML datastore
  azure.azcollection.azure_rm_ml_datastore_info:
    name: MyData
    resource_group: myResourceGroup
    ml_workspace: myMLWorkspace

- name: List ML datastore
  azure.azcollection.azure_rm_ml_datastore_info:
    resource_group: myResourceGroup
    ml_workspace: myMLWorkspace
'''

RETURN = '''
ml_datastore:
    description:
        - Datastore that was just created or updated.
    returned: always
    type: dict
    sample: [
      {
        "account_name": "workspacstoragexxxxxxxxx",
        "container_name": "datastorexxxxxxxxx-xxxxxxxx-xxxx-xxxx-xxxx-xxxxxxxxxxxx",
        "credentials": {},
        "description": "here is a description",
        "endpoint": "core.windows.net",
        "id": "/subscriptions/xxxxxxxx-xxxx-xxxx-xxxx-xxxxxxxxxxxx/resourceGroups/xxxxx-ml-datastore/providers/Microsoft.MachineLearningServices/workspaces/workspaceexxxxxxxx/datastores/datastorexxxxxxxxx",
        "name": "datastorexxxxxxxxx",
        "protocol": "https",
        "tags": {},
        "type": "azure_blob",
      }
    ]
'''  # NOQA


try:
    from ansible_collections.azure.azcollection.plugins.module_utils.azure_rm_common_ml import MLClientCommon
    from azure.core.exceptions import ResourceNotFoundError
except ImportError:
    # This is handled in azure_rm_common
    pass


class AzureRMMLDatastoreInfo(MLClientCommon):
    def __init__(self):

        self.module_arg_spec = dict(
            name=dict(
                type='str',
                required=False,
            ),
            resource_group=dict(
                type='str',
                required=True,
            ),
            ml_workspace=dict(
                type='str',
                required=True,
            ),
        )

        self._client = None
        self.ml_registry = None

        self.results = dict(
            ml_datastores=[]
        )

        super(AzureRMMLDatastoreInfo, self).__init__(self.module_arg_spec,
                                                     supports_tags=False,
                                                     supports_check_mode=True,
                                                     facts_module=True,
                                                     )

    def exec_module(self, **kwargs):

        for key in list(self.module_arg_spec.keys()):
            setattr(self, key, kwargs[key])

        if self.name:
            try:
                result = self.client.datastores.get(name=self.name,
                                                    include_secrets=False)
                ml_datastores = [self.entity_to_dict(result)]
            except ResourceNotFoundError:
                ml_datastores = []
        else:
            results = self.client.datastores.list(include_secrets=False)

            ml_datastores = [self.entity_to_dict(x) for x in results]

        self.results['ml_datastores'] = ml_datastores
        return self.results


def main():
    AzureRMMLDatastoreInfo()


if __name__ == '__main__':
    main()



#!/usr/bin/python
# -*- coding: utf-8 -*-

# GNU General Public License v3.0+ (see COPYING or https://www.gnu.org/licenses/gpl-3.0.txt)

from __future__ import absolute_import, division, print_function
__metaclass__ = type

ANSIBLE_METADATA = {'metadata_version': '1.1',
                    'status': ['preview'],
                    'supported_by': 'community'}

DOCUMENTATION = r'''
---
module: intersight_network_connectivity_policy
short_description: Network Connectivity Policy configuration for Cisco Intersight
description:
  - Manages Network Connectivity Policy configuration on Cisco Intersight.
  - A policy to configure network connectivity settings including DNS and IPv6 on Cisco Intersight managed servers.
  - For more information see L(Cisco Intersight,https://intersight.com/apidocs/networkconfig/Policy/get/).
extends_documentation_fragment: intersight
options:
  state:
    description:
      - If C(present), will verify the resource is present and will create if needed.
      - If C(absent), will verify the resource is absent and will delete if needed.
    type: str
    choices: [present, absent]
    default: present
  organization:
    description:
      - The name of the Organization this resource is assigned to.
      - Profiles, Policies, and Pools that are created within a Custom Organization are applicable only to devices in the same Organization.
    type: str
    default: default
  name:
    description:
      - The name assigned to the Network Connectivity Policy.
      - The name must be between 1 and 64 alphanumeric characters, allowing special characters :-_.
    type: str
    required: true
  description:
    description:
      - The user-defined description for the Network Connectivity Policy.
      - Description can contain letters(a-z, A-Z), numbers(0-9), hyphen(-), period(.), colon(:), or an underscore(_).
    type: str
    aliases: [descr]
  tags:
    description:
      - List of tags in Key:<user-defined key> Value:<user-defined value> format.
    type: list
    elements: dict
  enable_dynamic_dns:
    description:
      - If enabled, updates the resource records to the DNS from Cisco IMC.
    type: bool
    default: false
  dynamic_dns_domain:
    description:
      - The domain name appended to a hostname for a Dynamic DNS (DDNS) update.
      - If left blank, only a hostname is sent to the DDNS update request.
      - This parameter is optional even when enable_dynamic_dns is true.
    type: str
  enable_ipv4_dns_from_dhcp:
    description:
      - Enable IPv4 DNS from DHCP.
      - If enabled, Cisco IMC retrieves the DNS server addresses from DHCP.
    type: bool
    default: false
  preferred_ipv4_dns_server:
    description:
      - IP address of the primary DNS server.
      - This is used when enable_ipv4_dns_from_dhcp is false.
    type: str
    default: "0.0.0.0"
  alternate_ipv4_dns_server:
    description:
      - IP address of the secondary DNS server.
      - This is used when enable_ipv4_dns_from_dhcp is false.
    type: str
    default: "0.0.0.0"
  enable_ipv6:
    description:
      - If enabled, allows to configure IPv6 properties.
    type: bool
    default: false
  enable_ipv6_dns_from_dhcp:
    description:
      - If enabled, Cisco IMC retrieves the DNS server addresses from DHCP.
      - Use DHCP field must be enabled for IPv6 in Cisco IMC to enable it.
    type: bool
    default: false
  preferred_ipv6_dns_server:
    description:
      - IP address of the primary DNS server.
    type: str
    default: "::"
  alternate_ipv6_dns_server:
    description:
      - IP address of the secondary DNS server.
    type: str
    default: "::"
author:
  - Ron Gershburg (@rgershbu)
'''

EXAMPLES = r'''
- name: Create a Network Connectivity Policy with Dynamic DNS enabled
  cisco.intersight.intersight_network_connectivity_policy:
    api_private_key: "{{ api_private_key }}"
    api_key_id: "{{ api_key_id }}"
    organization: "default"
    name: "DynamicDNS-Policy"
    description: "Network connectivity policy with Dynamic DNS"
    tags:
      - Key: "Site"
        Value: "DataCenter-A"
    enable_dynamic_dns: true
    dynamic_dns_domain: "company.com"
    enable_ipv4_dns_from_dhcp: true
    enable_ipv6: true
    enable_ipv6_dns_from_dhcp: true
    state: present

- name: Create a Network Connectivity Policy with static DNS servers
  cisco.intersight.intersight_network_connectivity_policy:
    api_private_key: "{{ api_private_key }}"
    api_key_id: "{{ api_key_id }}"
    name: "StaticDNS-Policy"
    description: "Network connectivity policy with static DNS servers"
    enable_dynamic_dns: false
    enable_ipv4_dns_from_dhcp: false
    preferred_ipv4_dns_server: "8.8.8.8"
    alternate_ipv4_dns_server: "8.8.4.4"
    enable_ipv6: true
    enable_ipv6_dns_from_dhcp: false
    preferred_ipv6_dns_server: "2001:4860:4860::8888"
    alternate_ipv6_dns_server: "2001:4860:4860::8844"
    state: present

- name: Create a basic Network Connectivity Policy with defaults
  cisco.intersight.intersight_network_connectivity_policy:
    api_private_key: "{{ api_private_key }}"
    api_key_id: "{{ api_key_id }}"
    name: "Basic-Network-Policy"
    description: "A basic network connectivity policy"
    state: present

- name: Delete a Network Connectivity Policy
  cisco.intersight.intersight_network_connectivity_policy:
    api_private_key: "{{ api_private_key }}"
    api_key_id: "{{ api_key_id }}"
    name: "DynamicDNS-Policy"
    state: absent
'''

RETURN = r'''
api_response:
  description: The API response output returned by the specified resource.
  returned: always
  type: dict
  sample:
    "api_response": {
        "Name": "test_network_connectivity_policy",
        "ObjectType": "networkconfig.Policy",
        "EnableDynamicDns": true,
        "DynamicDnsDomain": "company.com",
        "EnableIpv4dnsFromDhcp": true,
        "EnableIpv6": true,
        "EnableIpv6dnsFromDhcp": true,
        "Tags": [
            {
                "Key": "Site",
                "Value": "DataCenter-A"
            }
        ]
    }
'''


from ansible.module_utils.basic import AnsibleModule
from ansible_collections.cisco.intersight.plugins.module_utils.intersight import IntersightModule, intersight_argument_spec


def main():
    argument_spec = intersight_argument_spec.copy()
    argument_spec.update(
        state=dict(type='str', choices=['present', 'absent'], default='present'),
        organization=dict(type='str', default='default'),
        name=dict(type='str', required=True),
        description=dict(type='str', aliases=['descr']),
        tags=dict(type='list', elements='dict'),
        enable_dynamic_dns=dict(type='bool', default=False),
        dynamic_dns_domain=dict(type='str'),
        enable_ipv4_dns_from_dhcp=dict(type='bool', default=False),
        preferred_ipv4_dns_server=dict(type='str', default='0.0.0.0'),
        alternate_ipv4_dns_server=dict(type='str', default='0.0.0.0'),
        enable_ipv6=dict(type='bool', default=False),
        enable_ipv6_dns_from_dhcp=dict(type='bool', default=False),
        preferred_ipv6_dns_server=dict(type='str', default='::'),
        alternate_ipv6_dns_server=dict(type='str', default='::')
    )
    module = AnsibleModule(
        argument_spec,
        supports_check_mode=True
    )

    intersight = IntersightModule(module)
    intersight.result['api_response'] = {}
    intersight.result['trace_id'] = ''

    # Resource path used to configure policy
    resource_path = '/networkconfig/Policies'

    # Define API body used in compares or create
    intersight.api_body = {
        'Organization': {
            'Name': intersight.module.params['organization'],
        },
        'Name': intersight.module.params['name'],
        'EnableDynamicDns': intersight.module.params['enable_dynamic_dns'],
        'EnableIpv4dnsFromDhcp': intersight.module.params['enable_ipv4_dns_from_dhcp'],
        'EnableIpv6': intersight.module.params['enable_ipv6'],
    }

    if intersight.module.params['state'] == 'present':
        intersight.set_tags_and_description()

        # Add fields for static DNS servers if not using DHCP
        if not intersight.module.params['enable_ipv4_dns_from_dhcp']:
            intersight.api_body['PreferredIpv4dnsServer'] = intersight.module.params['preferred_ipv4_dns_server']
            intersight.api_body['AlternateIpv4dnsServer'] = intersight.module.params['alternate_ipv4_dns_server']

        # Add IPv6 fields if enabled
        if intersight.module.params['enable_ipv6']:
            intersight.api_body['EnableIpv6dnsFromDhcp'] = intersight.module.params['enable_ipv6_dns_from_dhcp']

            # Add fields for static DNS servers if not using DHCP for IPv6
            if not intersight.module.params['enable_ipv6_dns_from_dhcp']:
                intersight.api_body['PreferredIpv6dnsServer'] = intersight.module.params['preferred_ipv6_dns_server']
                intersight.api_body['AlternateIpv6dnsServer'] = intersight.module.params['alternate_ipv6_dns_server']

        # Add Dynamic DNS fields if enabled and domain is provided
        if intersight.module.params['enable_dynamic_dns'] and intersight.module.params['dynamic_dns_domain']:
            intersight.api_body['DynamicDnsDomain'] = intersight.module.params['dynamic_dns_domain']

    intersight.configure_policy_or_profile(resource_path=resource_path)

    module.exit_json(**intersight.result)


if __name__ == '__main__':
    main()



#!/usr/bin/python
# Copyright (c) 2017, Arie Bregman <abregman@redhat.com>
# GNU General Public License v3.0+ (see LICENSES/GPL-3.0-or-later.txt or https://www.gnu.org/licenses/gpl-3.0.txt)
# SPDX-License-Identifier: GPL-3.0-or-later

from __future__ import annotations

DOCUMENTATION = r"""
module: ip_netns
author: "Arie Bregman (@bregman-arie)"
short_description: Manage network namespaces
requirements: [ip]
description:
  - Create or delete network namespaces using the C(ip) command.
extends_documentation_fragment:
  - community.general._attributes
attributes:
  check_mode:
    support: full
  diff_mode:
    support: none
options:
  name:
    description:
      - Name of the namespace.
    type: str
  state:
    default: "present"
    choices: [present, absent]
    description:
      - Whether the namespace should exist.
    type: str
"""

EXAMPLES = r"""
- name: Create a namespace named mario
  community.general.ip_netns:
    name: mario
    state: present

- name: Delete a namespace named luigi
  community.general.ip_netns:
    name: luigi
    state: absent
"""

RETURN = r"""
# Default return values
"""

from ansible.module_utils.basic import AnsibleModule


class Namespace:
    """Interface to network namespaces."""

    def __init__(self, module):
        self.module = module
        self.name = module.params["name"]
        self.state = module.params["state"]

    def _netns(self, command):
        """Run ip nents command"""
        return self.module.run_command(["ip", "netns"] + command)

    def exists(self):
        """Check if the namespace already exists"""
        rc, out, err = self.module.run_command(["ip", "netns", "list"])
        if rc != 0:
            self.module.fail_json(msg=f"{err}")
        return self.name in out

    def add(self):
        """Create network namespace"""
        rtc, out, err = self._netns(["add", self.name])

        if rtc != 0:
            self.module.fail_json(msg=err)

    def delete(self):
        """Delete network namespace"""
        rtc, out, err = self._netns(["del", self.name])
        if rtc != 0:
            self.module.fail_json(msg=err)

    def check(self):
        """Run check mode"""
        changed = False

        if self.state == "present" and self.exists():
            changed = True

        elif self.state == "absent" and self.exists():
            changed = True
        elif self.state == "present" and not self.exists():
            changed = True

        self.module.exit_json(changed=changed)

    def run(self):
        """Make the necessary changes"""
        changed = False

        if self.state == "absent":
            if self.exists():
                self.delete()
                changed = True
        elif self.state == "present":
            if not self.exists():
                self.add()
                changed = True

        self.module.exit_json(changed=changed)


def main():
    """Entry point."""
    module = AnsibleModule(
        argument_spec={
            "name": {"default": None},
            "state": {"default": "present", "choices": ["present", "absent"]},
        },
        supports_check_mode=True,
    )
    module.run_command_environ_update = {"LANGUAGE": "C", "LC_ALL": "C"}

    network_namespace = Namespace(module)
    if module.check_mode:
        network_namespace.check()
    else:
        network_namespace.run()


if __name__ == "__main__":
    main()



#!/usr/bin/python
# -*- coding: utf-8 -*-

# Copyright: (c) 2023, Akini Ross (@akinross) <akinross@cisco.com>
# GNU General Public License v3.0+ (see LICENSE or https://www.gnu.org/licenses/gpl-3.0.txt)

from __future__ import absolute_import, division, print_function

__metaclass__ = type

ANSIBLE_METADATA = {
    "metadata_version": "1.1",
    "status": ["preview"],
    "supported_by": "certified",
}

DOCUMENTATION = r"""
---
module: aci_l3out_to_sr_mpls_infra_l3out
short_description: Manage Layer 3 Outside (L3Out) to SR-MPLS Infra L3Outs objects (l3ext:ConsLbl)
description:
- Manage Layer 3 Outside (L3Out) to SR-MPLS Infra L3Outs objects on Cisco ACI fabrics.
options:
  tenant:
    description:
    - The name of an existing tenant.
    type: str
    aliases: [ tenant_name ]
  l3out:
    description:
    - The name of an existing SR MPLS VRF L3Out.
    type: str
    aliases: [ l3out_name, name ]
  infra_l3out:
    description:
    - The name of an existing SR-MPLS Infra L3Out.
    type: str
    aliases: [ infra_l3out_name ]
  external_epg:
    description:
    - The distinguished name (DN) of the external EPG.
    type: str
    aliases: [ external_epg_dn ]
  outbound_route_map:
    description:
    - The distinguished name (DN) of the outbound route map.
    type: str
    aliases: [ outbound_route_map_dn, outbound ]
  inbound_route_map:
    description:
    - The distinguished name (DN) of the inbound route map.
    - Use an empty string to remove the inbound route map.
    type: str
    aliases: [ inbound_route_map_dn, inbound ]
  description:
    description:
    - Description for the L3Out.
    type: str
    aliases: [ descr ]
  state:
    description:
    - Use C(present) or C(absent) for adding or removing.
    - Use C(query) for listing an object or multiple objects.
    type: str
    choices: [ absent, present, query ]
    default: present
  name_alias:
    description:
    - The alias for the current object. This relates to the nameAlias field in ACI.
    type: str
extends_documentation_fragment:
- cisco.aci.aci
- cisco.aci.annotation
- cisco.aci.owner

notes:
- The C(tenant) and C(l3out) used must exist before using this module in your playbook.
- The M(cisco.aci.aci_tenant) and M(cisco.aci.aci_l3out) modules can be used for this.
seealso:
- module: cisco.aci.aci_tenant
- module: cisco.aci.aci_l3out
- name: APIC Management Information Model reference
  description: More information about the internal APIC class B(l3ext:ConsLbl).
  link: https://developer.cisco.com/docs/apic-mim-ref/
author:
- Akini Ross (@akinross)
"""

EXAMPLES = r"""
- name: Add a new l3out to sr-mpls infra l3out
  cisco.aci.aci_l3out_to_sr_mpls_infra_l3out:
    host: apic
    username: admin
    password: SomeSecretPassword
    tenant: production
    l3out: prod_l3out
    description: L3Out for Production tenant
    infra_l3out: infra_l3out_name
    external_epg: uni/tn-production/out-l3out_name/instP-external_epg_name
    outbound_route_map: uni/tn-production/prof-outbound_route_map_name
    inbound_route_map: uni/tn-production/prof-inbound_route_map_name
    state: present
  delegate_to: localhost

- name: Delete a l3out to sr-mpls infra l3out
  cisco.aci.aci_l3out_to_sr_mpls_infra_l3out:
    host: apic
    username: admin
    password: SomeSecretPassword
    tenant: production
    l3out: prod_l3out
    infra_l3out: infra_l3out_name
    state: absent
  delegate_to: localhost

- name: Query a l3out to sr-mpls infra l3out
  cisco.aci.aci_l3out_to_sr_mpls_infra_l3out:
    host: apic
    username: admin
    password: SomeSecretPassword
    tenant: production
    l3out: prod_l3out
    state: query
  delegate_to: localhost
  register: query_result

- name: Query all l3out to sr-mpls infra l3outs
  cisco.aci.aci_l3out_to_sr_mpls_infra_l3out:
    host: apic
    username: admin
    password: SomeSecretPassword
    state: query
  delegate_to: localhost
  register: query_all_result
"""

RETURN = r"""
current:
  description: The existing configuration from the APIC after the module has finished
  returned: success
  type: list
  sample:
    [
        {
            "fvTenant": {
                "attributes": {
                    "descr": "Production environment",
                    "dn": "uni/tn-production",
                    "name": "production",
                    "nameAlias": "",
                    "ownerKey": "",
                    "ownerTag": ""
                }
            }
        }
    ]
error:
  description: The error information as returned from the APIC
  returned: failure
  type: dict
  sample:
    {
        "code": "122",
        "text": "unknown managed object class foo"
    }
raw:
  description: The raw output returned by the APIC REST API (xml or json)
  returned: parse error
  type: str
  sample: '<?xml version="1.0" encoding="UTF-8"?><imdata totalCount="1"><error code="122" text="unknown managed object class foo"/></imdata>'
sent:
  description: The actual/minimal configuration pushed to the APIC
  returned: info
  type: list
  sample:
    {
        "fvTenant": {
            "attributes": {
                "descr": "Production environment"
            }
        }
    }
previous:
  description: The original configuration from the APIC before the module has started
  returned: info
  type: list
  sample:
    [
        {
            "fvTenant": {
                "attributes": {
                    "descr": "Production",
                    "dn": "uni/tn-production",
                    "name": "production",
                    "nameAlias": "",
                    "ownerKey": "",
                    "ownerTag": ""
                }
            }
        }
    ]
proposed:
  description: The assembled configuration from the user-provided parameters
  returned: info
  type: dict
  sample:
    {
        "fvTenant": {
            "attributes": {
                "descr": "Production environment",
                "name": "production"
            }
        }
    }
filter_string:
  description: The filter string used for the request
  returned: failure or debug
  type: str
  sample: ?rsp-prop-include=config-only
method:
  description: The HTTP method used for the request to the APIC
  returned: failure or debug
  type: str
  sample: POST
response:
  description: The HTTP response from the APIC
  returned: failure or debug
  type: str
  sample: OK (30 bytes)
status:
  description: The HTTP status from the APIC
  returned: failure or debug
  type: int
  sample: 200
url:
  description: The HTTP url used for the request to the APIC
  returned: failure or debug
  type: str
  sample: https://10.11.12.13/api/mo/uni/tn-production.json
"""

from ansible.module_utils.basic import AnsibleModule
from ansible_collections.cisco.aci.plugins.module_utils.aci import ACIModule, aci_argument_spec, aci_annotation_spec, aci_owner_spec


def main():
    argument_spec = aci_argument_spec()
    argument_spec.update(aci_annotation_spec())
    argument_spec.update(aci_owner_spec())
    argument_spec.update(
        tenant=dict(type="str", aliases=["tenant_name"]),  # Not required for querying all objects
        l3out=dict(type="str", aliases=["l3out_name", "name"]),  # Not required for querying all objects
        description=dict(type="str", aliases=["descr"]),
        infra_l3out=dict(type="str", aliases=["infra_l3out_name"]),
        external_epg=dict(type="str", aliases=["external_epg_dn"]),
        outbound_route_map=dict(type="str", aliases=["outbound_route_map_dn", "outbound"]),
        inbound_route_map=dict(type="str", aliases=["inbound_route_map_dn", "inbound"]),
        state=dict(type="str", default="present", choices=["absent", "present", "query"]),
        name_alias=dict(type="str"),
    )

    module = AnsibleModule(
        argument_spec=argument_spec,
        supports_check_mode=True,
        required_if=[
            ["state", "absent", ["l3out", "tenant", "infra_l3out"]],
            ["state", "present", ["l3out", "tenant", "infra_l3out", "external_epg", "outbound_route_map"]],
        ],
    )

    aci = ACIModule(module)

    tenant = module.params.get("tenant")
    l3out = module.params.get("l3out")
    description = module.params.get("description")
    infra_l3out = module.params.get("infra_l3out")
    external_epg = module.params.get("external_epg")
    outbound_route_map = module.params.get("outbound_route_map")
    inbound_route_map = module.params.get("inbound_route_map")
    state = module.params.get("state")
    name_alias = module.params.get("name_alias")

    # l3extRsProvLblDef, bgpDomainIdAllocator are auto-generated classes, added for query output
    child_classes = ["l3extRsLblToInstP", "l3extRsLblToProfile", "l3extRsProvLblDef", "bgpDomainIdAllocator"]

    aci.construct_url(
        root_class=dict(
            aci_class="fvTenant",
            aci_rn="tn-{0}".format(tenant),
            module_object=tenant,
            target_filter={"name": tenant},
        ),
        subclass_1=dict(
            aci_class="l3extOut",
            aci_rn="out-{0}".format(l3out),
            module_object=l3out,
            target_filter={"name": l3out},
        ),
        subclass_2=dict(
            aci_class="l3extConsLbl",
            aci_rn="conslbl-{0}".format(infra_l3out),
            module_object=infra_l3out,
            target_filter={"name": infra_l3out},
        ),
        child_classes=child_classes,
    )

    aci.get_existing()

    if state == "present":
        child_configs = []

        if aci.existing:
            children = aci.existing[0].get("l3extConsLbl", {}).get("children", [])
            for child in children:
                if child.get("l3extRsLblToProfile"):
                    tdn = child.get("l3extRsLblToProfile").get("attributes").get("tDn")
                    direction = child.get("l3extRsLblToProfile").get("attributes").get("direction")
                    route_map = outbound_route_map if direction == "export" else inbound_route_map
                    # Inbound route-map is removed when input is different or an empty string, otherwise ignored.
                    if route_map is not None and tdn != route_map:
                        child_configs.append(dict(l3extRsLblToProfile=dict(attributes=dict(tDn=tdn, direction=direction, status="deleted"))))
                elif child.get("l3extRsLblToInstP"):
                    tdn = child.get("l3extRsLblToInstP").get("attributes").get("tDn")
                    if tdn != external_epg:
                        child_configs.append(dict(l3extRsLblToInstP=dict(attributes=dict(tDn=tdn, status="deleted"))))

        child_configs.append(dict(l3extRsLblToProfile=dict(attributes=dict(tDn=outbound_route_map, direction="export"))))
        child_configs.append(dict(l3extRsLblToInstP=dict(attributes=dict(tDn=external_epg))))

        if inbound_route_map:
            child_configs.append(dict(l3extRsLblToProfile=dict(attributes=dict(tDn=inbound_route_map, direction="import"))))

        aci.payload(
            aci_class="l3extConsLbl",
            class_config=dict(
                name=infra_l3out,
                descr=description,
                nameAlias=name_alias,
            ),
            child_configs=child_configs,
        )

        aci.get_diff(aci_class="l3extConsLbl")

        aci.post_config()

    elif state == "absent":
        aci.delete_config()

    aci.exit_json()


if __name__ == "__main__":
    main()



#!/usr/bin/python

# Copyright (c) 2026, Dexter Le <dextersydney2001@gmail.com>
# GNU General Public License v3.0+ (see LICENSES/GPL-3.0-or-later.txt or https://www.gnu.org/licenses/gpl-3.0.txt)
# SPDX-License-Identifier: GPL-3.0-or-later

from __future__ import annotations

DOCUMENTATION = r"""
module: kopia_repository_info
short_description: Gather information about a Kopia repository
author:
  - Dexter Le (@munchtoast)
version_added: "13.1.0"
description:
  - Gather read-only information about the current Kopia repository connection and throttle settings.
  - Runs C(kopia repository status) and C(kopia repository throttle get).
extends_documentation_fragment:
  - community.general._attributes
  - community.general._attributes.info_module
  - community.general._kopia
"""

EXAMPLES = r"""
- name: Gather Kopia repository info
  community.general.kopia_repository_info:
    config: /etc/kopia/root.config
  register: result

- name: Show repository status
  ansible.builtin.debug:
    msg: "{{ result.repository_status }}"

- name: Show throttle settings
  ansible.builtin.debug:
    msg: "{{ result.throttle }}"
"""

RETURN = r"""
repository_status:
  description: Output of C(kopia repository status).
  type: str
  returned: always
  sample: |-
    Connected to repository: s3:/my-bucket/
    Config file: /etc/kopia/root.config
    ...
throttle:
  description: Output of C(kopia repository throttle get) showing current throttle limits.
  type: str
  returned: always
  sample: |-
    upload-bytes-per-second: 0
    download-bytes-per-second: 0
"""

from ansible_collections.community.general.plugins.module_utils._kopia import (
    KOPIA_COMMON_ARGUMENT_SPEC,
    kopia_runner,
)
from ansible_collections.community.general.plugins.module_utils._module_helper import ModuleHelper


class KopiaRepositoryInfo(ModuleHelper):
    module = dict(
        argument_spec=dict(**KOPIA_COMMON_ARGUMENT_SPEC),
        supports_check_mode=True,
    )
    output_params = ["repository_status", "throttle"]

    def __init_module__(self):
        self.runner = kopia_runner(self.module)

    def _process_command_output(self, cli_action=""):
        def process(rc, out, err):
            if rc != 0:
                self.do_raise(f"kopia {cli_action} failed with error (rc={rc}): {err}")
            return out.rstrip() if out else None

        return process

    def __run__(self):
        with self.runner(
            "status config",
            output_process=self._process_command_output("repository status"),
        ) as ctx:
            self.vars.set(
                "repository_status",
                ctx.run(),
                output=True,
            )

        with self.runner(
            "get_throttle config",
            output_process=self._process_command_output("repository throttle get"),
        ) as ctx:
            self.vars.set(
                "throttle",
                ctx.run(),
                output=True,
            )


def main():
    KopiaRepositoryInfo.execute()


if __name__ == "__main__":
    main()



#!/usr/bin/python
# Copyright (c) 2016-2017, Yanis Guenane <yanis+ansible@guenane.org>
# Copyright (c) 2017, Markus Teufelberger <mteufelberger+ansible@mgit.at>
# GNU General Public License v3.0+ (see LICENSES/GPL-3.0-or-later.txt or https://www.gnu.org/licenses/gpl-3.0.txt)
# SPDX-License-Identifier: GPL-3.0-or-later

from __future__ import annotations

DOCUMENTATION = r"""
module: x509_certificate_info
short_description: Provide information of OpenSSL X.509 certificates
description:
  - This module allows one to query information on OpenSSL certificates.
  - It uses the cryptography Python library to interact with OpenSSL.
  - Note that this module was called C(openssl_certificate_info) when included directly in Ansible up to version 2.9. When
    moved to the collection C(community.crypto), it was renamed to M(community.crypto.x509_certificate_info). From Ansible
    2.10 on, it can still be used by the old short name (or by C(ansible.builtin.openssl_certificate_info)), which redirects
    to M(community.crypto.x509_certificate_info). When using FQCNs or when using the
    L(collections,https://docs.ansible.com/ansible/latest/user_guide/collections_using.html#using-collections-in-a-playbook)
    keyword, the new name M(community.crypto.x509_certificate_info) should be used to avoid a deprecation warning.
author:
  - Felix Fontein (@felixfontein)
  - Yanis Guenane (@Spredzy)
  - Markus Teufelberger (@MarkusTeufelberger)
extends_documentation_fragment:
  - community.crypto._attributes
  - community.crypto._attributes.info_module
  - community.crypto._attributes.idempotent_not_modify_state
  - community.crypto._cryptography_dep.minimum
  - community.crypto._name_encoding
options:
  path:
    description:
      - Remote absolute path where the certificate file is loaded from.
      - Either O(path) or O(content) must be specified, but not both.
      - PEM and DER formats are supported.
    type: path
  content:
    description:
      - Content of the X.509 certificate in PEM format.
      - Either O(path) or O(content) must be specified, but not both.
    type: str
    version_added: '1.0.0'
  valid_at:
    description:
      - A dict of names mapping to time specifications. Every time specified here will be checked whether the certificate
        is valid at this point. See the RV(valid_at) return value for information on the result.
      - Time can be specified either as relative time or as absolute timestamp.
      - Time will always be interpreted as UTC.
      - Valid format is C([+-]timespec | ASN.1 TIME) where timespec can be an integer + C([w | d | h | m | s]) (for example
        V(+32w1d2h)), and ASN.1 TIME (in other words, pattern C(YYYYMMDDHHMMSSZ)). Note that all timestamps will be treated
        as being in UTC.
    type: dict
  select_crypto_backend:
    description:
      - Determines which crypto backend to use.
      - The default choice is V(auto), which tries to use C(cryptography) if available.
      - If set to V(cryptography), will try to use the L(cryptography,https://cryptography.io/) library.
      - Note that with community.crypto 3.0.0, all values behave the same.
        This option will be deprecated in a later version.
        We recommend to not set it explicitly.
    type: str
    default: auto
    choices: [auto, cryptography]

notes:
  - All timestamp values are provided in ASN.1 TIME format, in other words, following the C(YYYYMMDDHHMMSSZ) pattern. They
    are all in UTC.
seealso:
  - module: community.crypto.x509_certificate
  - module: community.crypto.x509_certificate_pipe
  - plugin: community.crypto.x509_certificate_info
    plugin_type: filter
    description: A filter variant of this module.
  - plugin: community.crypto.to_serial
    plugin_type: filter
"""

EXAMPLES = r"""
---
- name: Generate a Self Signed OpenSSL certificate
  community.crypto.x509_certificate:
    path: /etc/ssl/crt/ansible.com.crt
    privatekey_path: /etc/ssl/private/ansible.com.pem
    csr_path: /etc/ssl/csr/ansible.com.csr
    provider: selfsigned


# Get information on the certificate

- name: Get information on generated certificate
  community.crypto.x509_certificate_info:
    path: /etc/ssl/crt/ansible.com.crt
  register: result

- name: Dump information
  ansible.builtin.debug:
    var: result


# Check whether the certificate is valid or not valid at certain times, fail
# if this is not the case. The first task (x509_certificate_info) collects
# the information, and the second task (assert) validates the result and
# makes the playbook fail in case something is not as expected.

- name: Test whether that certificate is valid tomorrow and/or in three weeks
  community.crypto.x509_certificate_info:
    path: /etc/ssl/crt/ansible.com.crt
    valid_at:
      point_1: "+1d"
      point_2: "+3w"
  register: result

- name: Validate that certificate is valid tomorrow, but not in three weeks
  ansible.builtin.assert:
    that:
      - result.valid_at.point_1    # valid in one day
      - not result.valid_at.point_2 # not valid in three weeks
"""

RETURN = r"""
expired:
  description: Whether the certificate is expired (in other words, C(notAfter) is in the past).
  returned: success
  type: bool
basic_constraints:
  description: Entries in the C(basic_constraints) extension, or V(none) if extension is not present.
  returned: success
  type: list
  elements: str
  sample: ["CA:TRUE", "pathlen:1"]
basic_constraints_critical:
  description: Whether the C(basic_constraints) extension is critical.
  returned: success
  type: bool
extended_key_usage:
  description: Entries in the C(extended_key_usage) extension, or V(none) if extension is not present.
  returned: success
  type: list
  elements: str
  sample: [Biometric Info, DVCS, Time Stamping]
extended_key_usage_critical:
  description: Whether the C(extended_key_usage) extension is critical.
  returned: success
  type: bool
extensions_by_oid:
  description: Returns a dictionary for every extension OID.
  returned: success
  type: dict
  contains:
    critical:
      description: Whether the extension is critical.
      returned: success
      type: bool
    value:
      description:
        - The Base64 encoded value (in DER format) of the extension.
        - B(Note) that depending on the C(cryptography) version used, it is not possible to extract the ASN.1 content of the
          extension, but only to provide the re-encoded content of the extension in case it was parsed by C(cryptography).
          This should usually result in exactly the same value, except if the original extension value was malformed.
      returned: success
      type: str
      sample: "MAMCAQU="
  sample: {"1.3.6.1.5.5.7.1.24": {"critical": false, "value": "MAMCAQU="}}
key_usage:
  description: Entries in the C(key_usage) extension, or V(none) if extension is not present.
  returned: success
  type: str
  sample: [Key Agreement, Data Encipherment]
key_usage_critical:
  description: Whether the C(key_usage) extension is critical.
  returned: success
  type: bool
subject_alt_name:
  description:
    - Entries in the C(subject_alt_name) extension, or V(none) if extension is not present.
    - See O(name_encoding) for how IDNs are handled.
  returned: success
  type: list
  elements: str
  sample: ["DNS:www.ansible.com", "IP:1.2.3.4"]
subject_alt_name_critical:
  description: Whether the C(subject_alt_name) extension is critical.
  returned: success
  type: bool
ocsp_must_staple:
  description: V(true) if the OCSP Must Staple extension is present, V(none) otherwise.
  returned: success
  type: bool
ocsp_must_staple_critical:
  description: Whether the C(ocsp_must_staple) extension is critical.
  returned: success
  type: bool
issuer:
  description:
    - The certificate's issuer.
    - Note that for repeated values, only the last one will be returned.
  returned: success
  type: dict
  sample: {"organizationName": "Ansible", "commonName": "ca.example.com"}
issuer_ordered:
  description: The certificate's issuer as an ordered list of tuples.
  returned: success
  type: list
  elements: list
  sample: [["organizationName", "Ansible"], ["commonName": "ca.example.com"]]
subject:
  description:
    - The certificate's subject as a dictionary.
    - Note that for repeated values, only the last one will be returned.
  returned: success
  type: dict
  sample: {"commonName": "www.example.com", "emailAddress": "test@example.com"}
subject_ordered:
  description: The certificate's subject as an ordered list of tuples.
  returned: success
  type: list
  elements: list
  sample: [["commonName", "www.example.com"], ["emailAddress": "test@example.com"]]
not_after:
  description: C(notAfter) date as ASN.1 TIME.
  returned: success
  type: str
  sample: '20190413202428Z'
not_before:
  description: C(notBefore) date as ASN.1 TIME.
  returned: success
  type: str
  sample: '20190331202428Z'
public_key:
  description: Certificate's public key in PEM format.
  returned: success
  type: str
  sample: "-----BEGIN PUBLIC KEY-----\nMIICIjANBgkqhkiG9w0BAQEFAAOCAg8A..."
public_key_type:
  description:
    - The certificate's public key's type.
    - One of V(RSA), V(DSA), V(ECC), V(Ed25519), V(X25519), V(Ed448), or V(X448).
    - Will start with V(unknown) if the key type cannot be determined.
  returned: success
  type: str
  version_added: 1.7.0
  sample: RSA
public_key_data:
  description:
    - Public key data. Depends on the public key's type.
  returned: success
  type: dict
  version_added: 1.7.0
  contains:
    size:
      description:
        - Bit size of modulus (RSA) or prime number (DSA).
      type: int
      returned: When RV(public_key_type=RSA) or RV(public_key_type=DSA)
    modulus:
      description:
        - The RSA key's modulus.
      type: int
      returned: When RV(public_key_type=RSA)
    exponent:
      description:
        - The RSA key's public exponent.
      type: int
      returned: When RV(public_key_type=RSA)
    p:
      description:
        - The C(p) value for DSA.
        - This is the prime modulus upon which arithmetic takes place.
      type: int
      returned: When RV(public_key_type=DSA)
    q:
      description:
        - The C(q) value for DSA.
        - This is a prime that divides C(p - 1), and at the same time the order of the subgroup of the multiplicative group
          of the prime field used.
      type: int
      returned: When RV(public_key_type=DSA)
    g:
      description:
        - The C(g) value for DSA.
        - This is the element spanning the subgroup of the multiplicative group of the prime field used.
      type: int
      returned: When RV(public_key_type=DSA)
    curve:
      description:
        - The curve's name for ECC.
      type: str
      returned: When RV(public_key_type=ECC)
    exponent_size:
      description:
        - The maximum number of bits of a private key. This is basically the bit size of the subgroup used.
      type: int
      returned: When RV(public_key_type=ECC)
    x:
      description:
        - The C(x) coordinate for the public point on the elliptic curve.
      type: int
      returned: When RV(public_key_type=ECC)
    y:
      description:
        - For RV(public_key_type=ECC), this is the C(y) coordinate for the public point on the elliptic curve.
        - For RV(public_key_type=DSA), this is the publicly known group element whose discrete logarithm w.r.t. C(g) is the
          private key.
      type: int
      returned: When RV(public_key_type=DSA) or RV(public_key_type=ECC)
public_key_fingerprints:
  description:
    - Fingerprints of certificate's public key.
    - For every hash algorithm available, the fingerprint is computed.
  returned: success
  type: dict
  sample: "{'sha256': 'd4:b3:aa:6d:c8:04:ce:4e:ba:f6:29:4d:92:a3:94:b0:c2:ff:bd:bf:33:63:11:43:34:0f:51:b0:95:09:2f:63', 'sha512':
    'f7:07:4a:f0:b0:f0:e6:8b:95:5f:f9:e6:61:0a:32:68:f1..."
fingerprints:
  description:
    - Fingerprints of the DER-encoded form of the whole certificate.
    - For every hash algorithm available, the fingerprint is computed.
  returned: success
  type: dict
  sample: "{'sha256': 'd4:b3:aa:6d:c8:04:ce:4e:ba:f6:29:4d:92:a3:94:b0:c2:ff:bd:bf:33:63:11:43:34:0f:51:b0:95:09:2f:63', 'sha512':
    'f7:07:4a:f0:b0:f0:e6:8b:95:5f:f9:e6:61:0a:32:68:f1..."
  version_added: 1.2.0
signature_algorithm:
  description: The signature algorithm used to sign the certificate.
  returned: success
  type: str
  sample: sha256WithRSAEncryption
serial_number:
  description:
    - The certificate's serial number.
    - This return value is an B(integer). If you need the serial numbers as a colon-separated hex string, such as C(11:22:33),
      you need to convert it to that form with P(community.crypto.to_serial#filter).
  returned: success
  type: int
  sample: 1234
version:
  description: The certificate version.
  returned: success
  type: int
  sample: 3
valid_at:
  description: For every time stamp provided in the O(valid_at) option, a boolean whether the certificate is valid at that
    point in time or not.
  returned: success
  type: dict
subject_key_identifier:
  description:
    - The certificate's subject key identifier.
    - The identifier is returned in hexadecimal, with V(:) used to separate bytes.
    - Is V(none) if the C(SubjectKeyIdentifier) extension is not present.
  returned: success
  type: str
  sample: '00:11:22:33:44:55:66:77:88:99:aa:bb:cc:dd:ee:ff:00:11:22:33'
authority_key_identifier:
  description:
    - The certificate's authority key identifier.
    - The identifier is returned in hexadecimal, with V(:) used to separate bytes.
    - Is V(none) if the C(AuthorityKeyIdentifier) extension is not present.
  returned: success
  type: str
  sample: '00:11:22:33:44:55:66:77:88:99:aa:bb:cc:dd:ee:ff:00:11:22:33'
authority_cert_issuer:
  description:
    - The certificate's authority cert issuer as a list of general names.
    - Is V(none) if the C(AuthorityKeyIdentifier) extension is not present.
    - See O(name_encoding) for how IDNs are handled.
  returned: success
  type: list
  elements: str
  sample: ["DNS:www.ansible.com", "IP:1.2.3.4"]
authority_cert_serial_number:
  description:
    - The certificate's authority cert serial number.
    - Is V(none) if the C(AuthorityKeyIdentifier) extension is not present.
    - This return value is an B(integer). If you need the serial numbers as a colon-separated hex string, such as C(11:22:33),
      you need to convert it to that form with P(community.crypto.to_serial#filter).
  returned: success
  type: int
  sample: 12345
ocsp_uri:
  description: The OCSP responder URI, if included in the certificate. Will be V(none) if no OCSP responder URI is included.
  returned: success
  type: str
issuer_uri:
  description: The Issuer URI, if included in the certificate. Will be V(none) if no issuer URI is included.
  returned: success
  type: str
  version_added: 2.9.0
"""

import typing as t

from ansible.module_utils.basic import AnsibleModule
from ansible.module_utils.common.text.converters import to_text

from ansible_collections.community.crypto.plugins.module_utils._crypto.basic import (
    OpenSSLObjectError,
)
from ansible_collections.community.crypto.plugins.module_utils._crypto.cryptography_support import (
    CRYPTOGRAPHY_TIMEZONE,
)
from ansible_collections.community.crypto.plugins.module_utils._crypto.module_backends.certificate_info import (
    select_backend,
)
from ansible_collections.community.crypto.plugins.module_utils._time import (
    get_relative_time_option,
)


def main() -> t.NoReturn:
    module = AnsibleModule(
        argument_spec={
            "path": {"type": "path"},
            "content": {"type": "str"},
            "valid_at": {"type": "dict"},
            "name_encoding": {
                "type": "str",
                "default": "ignore",
                "choices": ["ignore", "idna", "unicode"],
            },
            "select_crypto_backend": {
                "type": "str",
                "default": "auto",
                "choices": ["auto", "cryptography"],
            },
        },
        required_one_of=(["path", "content"],),
        mutually_exclusive=(["path", "content"],),
        supports_check_mode=True,
    )

    content: str | None = module.params["content"]
    path: str | None = module.params["path"]
    if content is not None:
        data = content.encode("utf-8")
    else:
        if path is None:
            module.fail_json(msg="One of path and content must be provided")
        try:
            with open(path, "rb") as f:
                data = f.read()
        except (IOError, OSError) as e:
            module.fail_json(msg=f"Error while reading certificate file from disk: {e}")

    module_backend = select_backend(module=module, content=data)

    valid_at: dict[str, t.Any] = module.params["valid_at"]
    if valid_at:
        for k, v in valid_at.items():
            if not isinstance(v, (str, bytes)):
                module.fail_json(
                    msg=f"The value for valid_at.{k} must be of type string (got {type(v)})"
                )
            valid_at[k] = get_relative_time_option(
                to_text(v),
                input_name=f"valid_at.{k}",
                with_timezone=CRYPTOGRAPHY_TIMEZONE,
            )

    try:
        result = module_backend.get_info(der_support_enabled=content is None)

        not_before = module_backend.get_not_before()
        not_after = module_backend.get_not_after()

        result["valid_at"] = {}
        if valid_at:
            for k, v in valid_at.items():
                result["valid_at"][k] = not_before <= v <= not_after

        module.exit_json(**result)
    except OpenSSLObjectError as exc:
        module.fail_json(msg=str(exc))


if __name__ == "__main__":
    main()
